package apap.tutorial.belajarbelajar.controller;

import apap.tutorial.belajarbelajar.model.CourseModel;
import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.model.PenyelenggaraModel;
import apap.tutorial.belajarbelajar.service.CourseService;
import apap.tutorial.belajarbelajar.service.PenyelenggaraService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import apap.tutorial.belajarbelajar.model.CourseModel;
import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.service.CourseService;
import apap.tutorial.belajarbelajar.service.PengajarService;


@Controller
public class CourseController {
    @Qualifier("courseServiceImpl")
    @Autowired
    private CourseService courseService;
    @Qualifier("penyelenggaraServiceImpl")
    @Autowired
    private PenyelenggaraService penyelenggaraService;

    @Qualifier("pengajarServiceImpl")
    @Autowired
    private PengajarService pengajarService;

    @GetMapping("/course/add")
    public String addCourseFormPage(Model model) {
        CourseModel course = new CourseModel();
        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();
        List<PenyelenggaraModel> listPenyelenggaraNew = new ArrayList<PenyelenggaraModel>();

        course.setListPenyelenggara(listPenyelenggaraNew);
        course.getListPenyelenggara().add(new PenyelenggaraModel());

        course.setListPengajar(new ArrayList<>());
        course.getListPengajar().add(new PengajarModel());

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);
        return "form-add-course";
    }

    @PostMapping(value="/course/add", params = {"save"})
    public String addCourseSubmit(@ModelAttribute CourseModel course, Model model) {
        if (course.getListPengajar() != null){
            for (PengajarModel pengajar : course.getListPengajar()){
                pengajar.setCourse(course);
            }
        }
        courseService.addCourse(course);
        model.addAttribute("code", course.getCode());
        return "add-course";
    }

    @PostMapping(value = "/course/add", params = {"addRow"})
    private String addRowPenyelenggaraMultiple(
            @ModelAttribute CourseModel course,
            Model model
    ) {
        if (course.getListPenyelenggara() == null || course.getListPenyelenggara().size() == 0) {
            course.setListPenyelenggara(new ArrayList<>());
        }
        course.getListPenyelenggara().add(new PenyelenggaraModel());
        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);

        return "form-add-course";
    }

    @PostMapping(value = "/course/add", params = {"deleteRow"})
    private String deleteRowPenyelenggaraMultiple(
            @ModelAttribute CourseModel course,
            @RequestParam("deleteRow") Integer row,
            Model model
    ) {
        final Integer rowId = Integer.valueOf(row);
        course.getListPenyelenggara().remove(rowId.intValue());

        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);

        return "form-add-course";
    }

    @PostMapping(value = "/course/add", params = {"addRowPengajar"})
    private String addRowPengajarMultiple(
            @ModelAttribute CourseModel course,
            Model model
    ) {
        if (course.getListPengajar() == null || course.getListPengajar().size() == 0) {
            course.setListPengajar(new ArrayList<>());
        }
        PengajarModel newPengajar = new PengajarModel();
        newPengajar.setCourse(course);
        course.getListPengajar().add(newPengajar);
        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);

        return "form-add-course";
    }

    @PostMapping(value = "/course/add", params = {"deleteRowPengajar"})
    private String deleteRowPengajarMultiple(
            @ModelAttribute CourseModel course,
            @RequestParam("deleteRowPengajar") Integer row,
            Model model
    ) {
        final Integer rowId = Integer.valueOf(row);
        course.getListPengajar().remove(rowId.intValue());

        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);

        return "form-add-course";
    }

    @GetMapping("/course/viewall")
    public String listCourse(Model model) {
        List<CourseModel> listCourse = courseService.getListCourse();
        model.addAttribute("listCourse", listCourse);
        return "viewall-course";
    }

    @GetMapping("/course/view")
    public String viewDetailCoursePage(@RequestParam(value = "code") String code, Model model) {
        CourseModel course = courseService.getCourseByCodeCourse(code);
        List<PengajarModel> listPengajar = course.getListPengajar();
        model.addAttribute("listPengajar", listPengajar);
        model.addAttribute("course", course);

        return "view-course";
    }

    @GetMapping("/course/view-query")
    public String viewDetailCoursePageQuery(@RequestParam(value = "code") String code, Model model) {
        CourseModel course = courseService.getCourseByCodeCourseQuery(code);
        List<PengajarModel> listPengajar = course.getListPengajar();
        model.addAttribute("listPengajar", listPengajar);
        model.addAttribute("course", course);

        return "view-course";
    }

    @GetMapping("/course/update/{code}")
    public String updateCourseFormPage(@PathVariable String code, Model model) {
        CourseModel course = courseService.getCourseByCodeCourse(code);
        model.addAttribute("course", course);

        return "form-update-course";
    }

    @PostMapping("/course/update")
    public String updateCourseSubmitPage(@ModelAttribute CourseModel course, Model model) {
        CourseModel updatedCourse = courseService.updateCourse(course);
        model.addAttribute("code", updatedCourse.getCode());

        return "update-course";
    }

    // Latihan no 1
    @GetMapping("/course/viewall-sortbyname")
    public String orderedListCourse(Model model) {
        List<CourseModel> listCourse = courseService.getListCourseAndSortByNameCourse();
        model.addAttribute("listCourse", listCourse);
        return "viewall-course";
    }

    // Latihan no 3
    @GetMapping("/course/{code}/delete-pengajar/{noPengajar}")
    public String updateCourseFormPage(@PathVariable String code, @PathVariable Long noPengajar, Model model) {
        CourseModel course = courseService.getCourseByCodeCourse(code);
        PengajarModel pengajar = pengajarService.getPengajarByNoPengajar(noPengajar);
        LocalDateTime dateTime = LocalDateTime.now();
        Boolean tutup = (dateTime.compareTo(course.getTanggalBerakhir()) > 0);

        if (tutup) {
            pengajarService.deletePengajar(pengajar);
            model.addAttribute("noPengajar", noPengajar);
            return "delete-pengajar";
        }
        model.addAttribute("code", code);

        return "course-belum-berakhir";
    }

    // latihan no 2
    @GetMapping("/pengajar/update/{noPengajar}")
    public String updatePengajarFormPage(@PathVariable Long noPengajar, Model model) {
        PengajarModel pengajar = pengajarService.getPengajarByNoPengajar(noPengajar);

        LocalDateTime sekarang = LocalDateTime.now();

        if (sekarang.isAfter(pengajar.getCourse().getTanggalBerakhir())) {
            model.addAttribute("pengajar", pengajar);
            return "form-update-pengajar";

        } else {
            return "gagal-update-pengajar";
        }

    }

    // latihan no 2
    @PostMapping("/pengajar/update")
    public String updatePengajarSubmitPage(@ModelAttribute PengajarModel pengajar, Model model) {
        PengajarModel updatedPengajar = pengajarService.updatePengajar(pengajar);
        model.addAttribute("noPengajar", updatedPengajar.getNoPengajar());

        return "update-pengajar";
    }

    // latihan no 4
    @GetMapping("/course/delete/{code}")
    public String deleteCourse(@PathVariable String code, Model model) {
        CourseModel course = courseService.getCourseByCodeCourse(code);
        LocalDateTime sekarang = LocalDateTime.now();

        if ((sekarang.isBefore(course.getTanggalBerakhir()) && sekarang.isAfter(course.getTanggalDimulai()))
                || course.getListPengajar().size() != 0) {
            return "gagal-delete-course";
        } else {
            courseService.deleteCourse(course);
            return "delete-course";
        }

    }
}
