package apap.tutorial.belajarbelajar.restcontroller;

import java.time.LocalDateTime;
import java.util.List;
import java.util.NoSuchElementException;

import javax.validation.Valid;

import apap.tutorial.belajarbelajar.model.CourseModel;
import apap.tutorial.belajarbelajar.rest.CourseDetail;
import apap.tutorial.belajarbelajar.service.PengajarRestService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.server.ResponseStatusException;

import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.service.PengajarRestService;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/api/v1")
public class PengajarRestController {
    @Autowired
    private PengajarRestService pengajarRestService;

    //Create pengajar
    @PostMapping(value = "/pengajar")
    private PengajarModel createPengajar(@Valid @RequestBody PengajarModel pengajar, BindingResult bindingResult){
        if(bindingResult.hasFieldErrors()){
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST, "Request body has invalid type or missing field."
            );
        } else {
            return pengajarRestService.createPengajar(pengajar);
        }
    }

    @PutMapping(value = "/pengajar/{noPengajar}")
    private PengajarModel updatePengajar(@PathVariable("noPengajar") Long noPengajar, @RequestBody PengajarModel pengajar){
        try{
            return pengajarRestService.updatePengajar(noPengajar, pengajar);
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                    HttpStatus.NOT_FOUND, "Pengajar dengan no " + noPengajar + " not found"
            );
        }
    }

    @GetMapping(value = "/pengajar/{noPengajar}")
    private PengajarModel getPengajarByNoPengajar(@PathVariable("noPengajar") Long noPengajar){
        try{
            return pengajarRestService.getPengajarByNoPengajar(noPengajar);
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                    HttpStatus.NOT_FOUND, "Pengajar dengan no " + noPengajar + " not found"
            );
        }
    }

    @GetMapping(value = "/list-pengajar")
    private List<PengajarModel> retrieveListPengajar(){
        return pengajarRestService.retrieveListPengajar();
    }

    @DeleteMapping(value = "pengajar/{noPengajar}")
    private ResponseEntity deletePengajar(@PathVariable("noPengajar") Long noPengajar){
        try{
            pengajarRestService.deletePengajar(noPengajar);
            return ResponseEntity.ok("Pengajar with no " + noPengajar + " has been deleted successfully");
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                    HttpStatus.NOT_FOUND, "Pengajar with no" + noPengajar + " not found"
            );
        } catch (UnsupportedOperationException e){
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST, "Pengajar cannot be deleted"
            );
        }
    }

    @GetMapping(value = "/pengajar/jenis-kelamin/{noPengajar}")
    private PengajarModel getStatus(@PathVariable("noPengajar") Long noPengajar){
        try{
            PengajarModel pengajar = pengajarRestService.getPengajarByNoPengajar(noPengajar); //Obj yang dikembaliin nanti
            LocalDateTime now = LocalDateTime.now();
            if (now.isBefore(pengajar.getCourse().getTanggalDimulai()) || now.isAfter(pengajar.getCourse().getTanggalBerakhir())){
                String retrievedMono = pengajarRestService.getPrediction(noPengajar); //masih dalam bentuk json
                String[] splitData = retrievedMono.split(","); //bentuk "gender":"female"
                String[] retGender = splitData[1].split(":"); //bentuk "gender" "female
                if(retGender[1].toLowerCase().equalsIgnoreCase("\"female\"")){
                    pengajar.setJenisKelamin(true);
                } else{
                    pengajar.setJenisKelamin(false);
                }
            }
            return pengajar;
        } catch (NoSuchElementException e){
            throw new ResponseStatusException(
                    HttpStatus.NOT_FOUND, "Pengajar with no" + noPengajar + " not found"
            );
        }




    }




}
