package apap.tutorial.belajarbelajar.service;
import java.time.LocalDateTime;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

import javax.transaction.Transactional;

import apap.tutorial.belajarbelajar.rest.CourseDetail;
import apap.tutorial.belajarbelajar.rest.Setting;
import apap.tutorial.belajarbelajar.rest.SettingPengajar;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.repository.CourseDb;
import apap.tutorial.belajarbelajar.service.PengajarService;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;
import apap.tutorial.belajarbelajar.model.CourseModel;
import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.repository.CourseDb;
import apap.tutorial.belajarbelajar.repository.PengajarDb;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

@Service
@Transactional
public class PengajarRestServiceImpl implements PengajarRestService{
    private final WebClient webClient;
    public PengajarRestServiceImpl(WebClient.Builder webClientBuilder){
        this.webClient = webClientBuilder.baseUrl(SettingPengajar.pengajarUrl).build();
    }
    @Autowired
    private PengajarDb pengajarDb;

    @Override
    public PengajarModel createPengajar(PengajarModel pengajar){
        return pengajarDb.save(pengajar);
    }

    @Override
    public PengajarModel getPengajarByNoPengajar(Long noPengajar){
        Optional<PengajarModel> pengajar = pengajarDb.findByNoPengajar(noPengajar);
        if (pengajar.isPresent()){
            return pengajar.get();
        } else {
            throw new NoSuchElementException();
        }
    }

    @Override
    public PengajarModel updatePengajar(Long noPengajar, PengajarModel pengajarUpdate){
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);
        //pengajar.setNoPengajar(pengajarUpdate.getNoPengajar());
        pengajar.setCourse(pengajarUpdate.getCourse());
        pengajar.setNamaPengajar(pengajarUpdate.getNamaPengajar());
        pengajar.setIsPengajarUniversitas(pengajarUpdate.getIsPengajarUniversitas());
        return pengajarDb.save(pengajar);
    }

    @Override
    public List<PengajarModel> retrieveListPengajar(){
        return pengajarDb.findAll();
    }

    @Override
    public void deletePengajar(Long noPengajar){
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);
        pengajarDb.delete(pengajar);

    }

    @Override
    public String getPrediction(Long noPengajar){
        PengajarModel pengajarObj = getPengajarByNoPengajar(noPengajar); //ambil obj pengajar dengan no pengajar
        String namaPengajar = pengajarObj.getNamaPengajar();
        String[] splitNamaPengajar = namaPengajar.split(" ");
        String nama = splitNamaPengajar[0];
        Mono<String> mono = this.webClient.get().uri("https://api.genderize.io/?name=" + nama)
                .retrieve()
                .bodyToMono(String.class);
        return mono.block();

    }




}
